/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "runanimationhandler.h"

#include "src/domain/domainfacade.h"


RunAnimationHandler::RunAnimationHandler ( QObject *parent, QPushButton *runAnimationButton, 
		QStatusBar *sb, const char *name ) : QObject(parent, name)
{
	this->runAnimationButton = runAnimationButton;
	this->statusBar = sb;
	this->timer = new QTimer(this);
	this->frameNr = 0;
	this->fps = 10;
}


void RunAnimationHandler::runAnimation()
{
	if( DomainFacade::getFacade()->getModelSize() != 0 ) {
		
		QObject::disconnect( runAnimationButton, SIGNAL(clicked()), this, SLOT(runAnimation()) );
		QObject::connect( runAnimationButton, SIGNAL(clicked()), this, SLOT(stopAnimation()) );
		runAnimationButton->setText( tr("Stop A&nimation") );
		runAnimationButton->setToggleButton(true);
		runAnimationButton->toggle();
		
		statusBar->message( tr("Running animation") );
		
		this->frameNr = 0;
		QObject::connect( timer, SIGNAL(timeout()), this, SLOT(playNextFrame()) );
		timer->start( 1000/fps, false );
	}
}


void RunAnimationHandler::stopAnimation()
{
	QObject::disconnect( runAnimationButton, SIGNAL(clicked()), this, SLOT(stopAnimation()) );
	QObject::connect(runAnimationButton, SIGNAL(clicked()), this, SLOT(runAnimation()));
	runAnimationButton->setText( tr("Run A&nimation") );
	runAnimationButton->setToggleButton(false);
	
	DomainFacade::getFacade()->setActiveFrame( frameNr );
	
	statusBar->clear();
	
	timer->stop();
}


void RunAnimationHandler::setSpeed(int fps)
{
	this->fps = fps;
	if( timer->isActive() ) {
		timer->changeInterval(1000/this->fps);
	}
}


void RunAnimationHandler::playNextFrame()
{
	DomainFacade::getFacade()->playFrame( frameNr );

	frameNr = (frameNr < DomainFacade::getFacade()->getModelSize() - 1) ? frameNr+1 : 0;
}
