/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ANIMATION_H
#define ANIMATION_H

#include "src/config.h"
#include "animationmodel.h"
#include "frame.h"

#include <vector>
#include <libxml/tree.h>

using namespace std;


/**
 *Implementation of the animationmodel containing the data about the animation.
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class Animation : public AnimationModel
{
public:
	Animation();
	~Animation();

	/**
	 *Inserts a new frame into the animationmodel.
	 *
	 *@param frameName the name of the frame to be inserted into the animationmodel
	 */
	void addFrame(char *frameName);
	
	
	/**
	 *Removes the active frame from the animation
	 */
	void removeFrame();
	
	
	/**
	 *Moves the frame at fromPosition from to toPosition.
	 *@param fromPosition the position the frame to be moved have.
	 *@param toPosition the position to move the frame to.
	 */
	void moveFrame(unsigned int fromPosition, unsigned int toPosition);
	
	
	/**
	 *Returns the frame with the number frameNumber.
	 *@param frameNumber the number of the frame to retrieve
	 *@return the frame with the number frameNumber.
	 */
	Frame* getFrame(unsigned int frameNumber);
	 
	
	/**
	 *Returns the size of the model.
	 *@return the size of the model.
	 */
	unsigned int getModelSize();
	
	
	/**
	 *Function for changing the currently active frame.
	 *@param frameNumber the number of the new active frame.
	 */
	void setActiveFrame(int frameNumber);

	
	/**
	  * Plays the frame with number frameNumber.
	  * @param frameNumber the number of the frame to play.
	  */
	void playFrame(int frameNumber);
	 
	 
	/**
	 * Returns the number of the currently active frame
	 * in the model.
	 * @return the number of the currently active frame.
	 */
	int getActiveFrameNumber();
	
	
	/**
	 * Clears the model.
	 */
	void clear();
	
	/**
	 * Opens a project.
	 */
	bool openProject(const char *filename);
	
	/**
	 * Saves the active project to a XML-file which is written to disk.
	 */
	bool saveProject(const char *directory);
	
	/**
	 * Creates a new project.
	 */
	bool newProject();
	
private:
	vector<Frame*> frames;
	int activeFrame;
	
	void getAttributes(xmlNode *node);
	bool saveDOMToFile(const char *directory, xmlDocPtr doc);
};

#endif
