/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "animationmodel.h"

#include "src/foundation/logger.h"


AnimationModel::~AnimationModel()
{
	/*int size = observers.size();
	for(unsigned int i=0; i<size; i++) {
		delete observers[i];
	}*/
}


void AnimationModel::attatch(Observer *o)
{
	if(o != NULL) {
		observers.push_back(o);
	}
	else {
		Logger::get().logDebug("Trying to attatch a NULL pointer to the AnimationModel");
	}
}


void AnimationModel::detatch(Observer *o)
{
	//Sequential search for the observer. Complexity: O(N/2) on the average.
	unsigned int size = observers.size();
	for(unsigned int i=0; i<size; i++) {
		if(&observers[i] == &o) {
			observers.erase(observers.begin() + i);
			break;
		}
	}
}


void AnimationModel::notifyAdd(Frame* frame, unsigned int index)
{
	for(unsigned int i=0; i<observers.size(); i++) {
		observers[i]->updateAdd(frame, index);
	}
}


void AnimationModel::notifyRemove(int frameNumber)
{
	Logger::get().logDebug("Notifying the observers of the removal of the frame");
	for(unsigned int i=0; i<observers.size(); i++) {
		observers[i]->updateRemove(frameNumber);
	}
}


void AnimationModel::notifyMove(unsigned int fromPosition, unsigned int toPosition)
{
	Logger::get().logDebug("Notifying the observers of the moving of the frame");
	for(unsigned int i=0; i<observers.size(); i++) {
		observers[i]->updateMove(fromPosition, toPosition);
	}
}


void AnimationModel::notifyNewActiveFrame(int frameNumber)
{
	Logger::get().logDebug("Notifying observers about new active frame");
	for(unsigned int i=0; i<observers.size(); i++) {
		observers[i]->updateNewActiveFrame(frameNumber);
	}
}

void AnimationModel::notifyClear()
{
	Logger::get().logDebug("Notifying observers that the model is erased");
	for(unsigned int i = 0; i < observers.size(); i++) {
		observers[i]->updateClear();
	}
}

void AnimationModel::notifyPlayFrame(int frameNumber)
{
	for(unsigned int i = 0; i < observers.size(); i++) {
		observers[i]->updatePlayFrame(frameNumber);
	}
}
