/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "domainfacade.h"

#include "animation/animation.h"
#include "src/foundation/logger.h"


DomainFacade::DomainFacade()
{
	animationModel = new Animation();
}


//Has to be here for getFacade to find domainFacade
DomainFacade* DomainFacade::domainFacade = 0;


DomainFacade* DomainFacade::getFacade()
{
	if(domainFacade == NULL) {
		domainFacade = new DomainFacade();
	}
	
	return domainFacade;
}


AnimationModel* DomainFacade::getModel()
{
	return animationModel;
}


void DomainFacade::addFrame(char *frameName)
{
	Logger::get().logDebug("Adding frame in the domainfacade");
	Logger::get().logDebug(frameName);
	animationModel->addFrame(frameName);
}


void DomainFacade::setActiveFrame(int frameNumber)
{
	animationModel->setActiveFrame(frameNumber);
}


int DomainFacade::getActiveFrameNumber()
{
	return animationModel->getActiveFrameNumber();
}


void DomainFacade::playFrame(int frameNumber)
{
	animationModel->playFrame(frameNumber);
}


void DomainFacade::removeFrame()
{
	animationModel->removeFrame();
}


bool DomainFacade::openProject(const char *filename)
{
	return animationModel->openProject(filename);
}


bool DomainFacade::saveProject(const char *directory)
{
	return animationModel->saveProject(directory);
}


bool DomainFacade::newProject()
{
	animationModel->clear();
	return true;
}


void DomainFacade::moveFrame(unsigned int fromPosition, unsigned int toPosition)
{
	animationModel->moveFrame(fromPosition, toPosition);
}


Frame* DomainFacade::getFrame(unsigned int frameNumber)
{
	return animationModel->getFrame(frameNumber);
}


int DomainFacade::getModelSize()
{
	return animationModel->getModelSize();
}
