/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "framebar.h"

#include "src/domain/domainfacade.h"
#include "thumbview.h"

#include <qlabel.h>
#include <qimage.h>
#include <qframe.h>



FrameBar::FrameBar(QWidget *parent, int height)
 : QScrollView(parent)
{
	//This line is uuuugly and might not work with different GUIs.
	this->FRAMEBAR_HEIGHT = height - this->horizontalScrollBar()->height()/2-5;
	
	activeFrame = -1;

	Logger::get().logDebug("FrameBar is attatched to the model"
		" and the model to framebar");
	animationModel = DomainFacade::getFacade()->getModel();
	animationModel->attatch(this);

	this->setMaximumHeight(115);
	this->setMinimumHeight(115);
	this->setHScrollBarMode ( AlwaysOn );
}


FrameBar::~FrameBar()
{
}


void FrameBar::updateAdd(Frame* frame, unsigned int index)
{
	Logger::get().logDebug("Uppdating the add in framebar");
	addFrame(frame, index);
}


void FrameBar::updateRemove(int frameNumber)
{
	Logger::get().logDebug("Recieving notification about the removal of a frame in the model");
	removeFrame(frameNumber);
}


void FrameBar::updateMove(unsigned int fromPosition, unsigned int toPosition)
{
	Logger::get().logDebug("Framebar recieves notification about the moving of a frame in the model");
	moveFrame(fromPosition, toPosition);
}


void FrameBar::updateNewActiveFrame(int frameNumber)
{
	setActiveFrame(frameNumber);
}


void FrameBar::updateClear()
{
	int size = thumbViews.size();
	for(unsigned int i = 0; i < (unsigned int)size; i++) {
		
		delete thumbViews[0];
		thumbViews.erase(thumbViews.begin());
	}
}


void FrameBar::updatePlayFrame(int)
{

}


/**
 *@todo evaluate if we should switch to ordinary scale instead of smoothscale for efficiency.
 */
void FrameBar::addFrame(Frame* frame, unsigned int index)
{
	if(index<=thumbViews.size()) {
		ThumbView *l = new ThumbView(this->viewport(), index);
		
		QImage i;
		i.load(frame->getPicture());
		QPixmap p;
		p.convertFromImage( i.scale(FRAMEBAR_HEIGHT-3, FRAMEBAR_HEIGHT) );
		l->setPixmap(p);
		
		l->setMinimumHeight(FRAMEBAR_HEIGHT);
		l->setMaximumHeight(FRAMEBAR_HEIGHT);
		l->setMinimumWidth(FRAMEBAR_HEIGHT);
		l->setMaximumWidth(FRAMEBAR_HEIGHT);
		
		if(index<thumbViews.size()) {
			for(unsigned int k=thumbViews.size()-1; k>=index; k--) {
				moveChild( thumbViews[k], childX(thumbViews[k])+FRAMEBAR_HEIGHT, 0 );
				thumbViews[k]->setFrameNumber(k+1);
			} 
		}
		
		thumbViews.insert(thumbViews.begin()+index, l);
		
		this->addChild(l, index*FRAMEBAR_HEIGHT);
		
		resizeContents(thumbViews.size()*FRAMEBAR_HEIGHT, FRAMEBAR_HEIGHT);
		l->show();
	}
}


void FrameBar::removeFrame(int frameNumber)
{
	if(frameNumber == activeFrame) {
		//TODO refactor this ifexpr by drawing out common code
		//The frame to be deleted is between other frames.
		if(frameNumber < (int)thumbViews.size() - 1) {
			//Move all frames behind the one deleted forward.
			for(int k = frameNumber + 1; k < (int)thumbViews.size(); k++) {
				moveChild( thumbViews[k], childX(thumbViews[k])-FRAMEBAR_HEIGHT, 0 );
				thumbViews[k]->setFrameNumber(k-1);
			}
			delete thumbViews[frameNumber];
			thumbViews.erase(thumbViews.begin() + frameNumber);
			//setActiveFrame(frameNumber);
		}
		//The frame to be deleted is the last frame.
		else {
			delete thumbViews[frameNumber];
			thumbViews.erase(thumbViews.begin() + frameNumber);
			//setActiveFrame(frameNumber - 1);
		}
	}
	else {	//TODO implement this when there is a need for removing frames which is not the active frame.
	
	}
	resizeContents(thumbViews.size()*FRAMEBAR_HEIGHT, FRAMEBAR_HEIGHT);
}


void FrameBar::moveFrame(unsigned int fromPosition, unsigned int toPosition)
{
	//Moving frame backward
	if(fromPosition > toPosition) {
		for(unsigned int k=toPosition; k<fromPosition; k++) {
			moveChild( thumbViews[k], childX(thumbViews[k]) + FRAMEBAR_HEIGHT, 0 );
			thumbViews[k]->setFrameNumber(k+1);
		}
		
		moveThumbView(fromPosition, toPosition);

		moveChild( thumbViews[toPosition], childX(thumbViews[toPosition]) -
				FRAMEBAR_HEIGHT*(fromPosition-toPosition), 0 );
	}
	//Moving frame forward
	else {
		for(unsigned int k=fromPosition + 1; k<=toPosition; k++) {
			moveChild( thumbViews[k], childX(thumbViews[k]) - FRAMEBAR_HEIGHT, 0 );
			thumbViews[k]->setFrameNumber(k-1);
		}
		
		moveThumbView(fromPosition, toPosition);
		
		moveChild( thumbViews[toPosition], childX(thumbViews[toPosition]) +
			FRAMEBAR_HEIGHT*(toPosition-fromPosition), 0 );
	}
}


void FrameBar::moveThumbView(unsigned int fromPosition, unsigned int toPosition)
{
	ThumbView *f = thumbViews[fromPosition];
	f->setFrameNumber(toPosition);
	thumbViews.erase(thumbViews.begin() + fromPosition);
	thumbViews.insert(thumbViews.begin() + toPosition, f );
}


void FrameBar::setActiveFrame(int frameNumber)
{
	if(frameNumber >= 0) {											//If there is a frame to set as active
		Logger::get().logDebug("Setting new active frame in FrameBar");
		if(activeFrame >= 0 && activeFrame < (int)thumbViews.size() ) {	//If there is an active frame who is painted
			thumbViews[activeFrame]->setFrameShape(QFrame::NoFrame);
		}
		
		thumbViews[frameNumber]->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
		thumbViews[frameNumber]->setLineWidth(4);
		thumbViews[frameNumber]->setMidLineWidth(4);
		
		this->center(frameNumber*FRAMEBAR_HEIGHT, 0, 0.0, 0.0);
	}
	activeFrame = frameNumber;
}
