
/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FRAMEBAR_H
#define FRAMEBAR_H

#include "src/config.h"
#include "src/presentation/observer.h"
#include "src/domain/animation/animationmodel.h"
#include "src/domain/animation/frame.h"
#include "thumbview.h"

#include <qscrollview.h>
#include <qhbox.h>

/**
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class FrameBar : public QScrollView, public Observer
{
	Q_OBJECT
public:
	FrameBar(QWidget *parent = 0, int height = 115);
	~FrameBar();
	
	/**
	 *Function to recieven notification when a frame is added.
	 */
	void updateAdd(Frame* frame, unsigned int index);
	
	/**
	 *Function to recieve notification when one or more frames are deleted.
	 *@param frameNumber the frame to remove
	 */
	void updateRemove(int frameNumber);
	
	/**
	 *Function to recieve notification when one or more frames are moved.
	 */
	void updateMove(unsigned int fromPosition, unsigned int toPosition);
	
	/**
	 *Function to recieve notification when a new frame is selected.
	 */
	void updateNewActiveFrame(int frameNumber);
	
	/**
	 * Function to receive notification when the model is erased.
	 */
	void updateClear();
	
	
	/**
	 * Function to recieve notification when a frame is to be played.
	 * @param frameNumber the frame to be played
	 */
	void updatePlayFrame(int frameNumber);
	
	 
private:
	AnimationModel *animationModel;	//Is this needed any longer with the push model?
	
	/**The height of the framebar*/
	int FRAMEBAR_HEIGHT;
	
	/**Vector of thumbviews to keep track of the pictures in the framebar*/
	vector<ThumbView*>thumbViews;
	
	/**The active frame in the framebar*/
	int activeFrame;

	
	/**
	 *Adds the picture to the frame to the framebar at position index.
	 *
	 *@param frame the path of the frame to be added to the framebar.
	 *@param index the place to add the frame.
	 */
	void addFrame(Frame* frame, unsigned int index);
	
	/**
	 *Removes the thumbview frameNumber from the framebar.
	 *@param frameNumber the frame to be removed from the framebar.
	 */
	void removeFrame(int frameNumber);
	
	/**
	 *Moves the thumbview at fromPosition in the framebar to toPosition
	 *@param fromPosition the position the thumbview to be moved is located at.
	 *@param toPosition the position to move the thumbview to.
	 */
	void moveFrame(unsigned int fromPosition, unsigned int toPosition);
	
	/**
	 *Sets the thumbview frameNumber in the framebar to be the active frame.
	 *@param frameNumber the number of the new active frame
	 */
	void setActiveFrame(int frameNumber);
	
	
	
	/**
	 *Moves a thumbview in the thumbviews-vector from fromPosition to toPosition.
	 *@param fromPosition the place in the vector the thumbview is moved from.
	 *@param toPosition the place in the vector the thumbview is moved to.
	 */
	void moveThumbView(unsigned int fromPosition, unsigned int toPosition);
};

#endif
