/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Most of this code is taken from an example program for Qt. Available at
// http://www.libsdl.org/cvs/qtSDL.tar.gz

#include "frameview.h"
#include "src/domain/domainfacade.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <qsize.h>
#if defined(Q_WS_X11)
#include <X11/Xlib.h>
#endif


FrameView::FrameView(QWidget *parent, const char *name)
 : QWidget(parent, name, WNoAutoErase), screen(NULL)
{
	Logger::get().logDebug("FrameView is attatched to the model"
		" and the model to FrameView");
	
	animationModel = DomainFacade::getFacade()->getModel();
	animationModel->attatch(this);	
	screen = NULL;
	currImage = NULL;
	widthConst  = 4;
	heightConst = 3;

	atexit(SDL_Quit);
	setNormalRatio();
}


FrameView::~FrameView()
{
	SDL_FreeSurface(currImage);
	SDL_FreeSurface(screen);
	currImage = NULL;
	screen = NULL;
}


void FrameView::setWidescreenRatio()
{
	widthConst  = 16;
	heightConst = 9;
	float minWidth = 300 * (16 / 9);
	this->setMinimumSize( (int)minWidth, 300);
	this->update();
}


void FrameView::setNormalRatio()
{
	widthConst  = 4;
	heightConst = 3;
	this->setMinimumSize(400, 300);
	this->update();
}


void FrameView::updateAdd(Frame*, unsigned int) 
{

}


void FrameView::updateRemove(int) 
{

}


void FrameView::updateMove(unsigned int, unsigned int) 
{

}


void FrameView::updateNewActiveFrame(int frameNumber)
{
	if (frameNumber > -1) {
		Frame *f = animationModel->getFrame(frameNumber);
		const char *str = f->getPicture();
		setActiveFrame(str);
	}
	else {
		SDL_FreeSurface(currImage);
		currImage = NULL;
		this->repaint();
	}
}


void FrameView::updatePlayFrame(int frameNumber)
{
	if (frameNumber > -1) {
		Frame *f = animationModel->getFrame(frameNumber);
		const char *str = f->getPicture();
		setActiveFrame(str);
	}
	else {
		SDL_FreeSurface(currImage);
		currImage = NULL;
		this->repaint();
	}
}


void FrameView::resizeEvent(QResizeEvent*)
{	
	char variable[64];
	// We could get a resize event at any time, so clean previous mode
	screen = NULL;
	SDL_QuitSubSystem(SDL_INIT_VIDEO);

	// Set the new video mode with the new window size
	sprintf(variable, "SDL_WINDOWID=0x%lx", winId());
	putenv(variable);
	if ( SDL_InitSubSystem(SDL_INIT_VIDEO) < 0 ) {
		char msg[80] = "Unable to init SDL: ";
		strcat( msg, SDL_GetError() );
		Logger::get().logFatal(msg);
    }
	
	screen = SDL_SetVideoMode(width(), height(), 0, 0);
	if (!screen) {
		char msg[80] = "Unable to set vide mode: ";
		strcat( msg, SDL_GetError() );
		Logger::get().logFatal(msg);
	}
}


void FrameView::paintEvent(QPaintEvent *)
{
#if defined(Q_WS_X11)
	// Make sure we're not conflicting with drawing from the Qt library
	XSync(QPaintDevice::x11Display(), FALSE);
#endif
	if (screen) {
		SDL_FillRect(screen, NULL, 0);
		
		if ( currImage ) {
			SDL_Rect dst;
			dst.x = (screen->w - currImage->w) / 2;
			dst.y = (screen->h - currImage->h) / 2;
			dst.w = currImage->w;
			dst.h = currImage->h;
			SDL_BlitSurface(currImage, NULL, screen, &dst);
		}
		SDL_Flip(screen);
	}
}


void FrameView::setActiveFrame(const char *fileName)
{
	Logger::get().logDebug("Setting new active frame in FrameView");
	if (currImage) {
		SDL_FreeSurface(currImage);
	}
	Logger::get().logDebug("Loading image");
	currImage = IMG_Load(fileName);
	Logger::get().logDebug("Loading image finished");
	this->update();
}


void FrameView::updateClear()
{
	SDL_FreeSurface(currImage);
	currImage = NULL;
	this->update();
}
