/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FRAMEVIEW_H
#define FRAMEVIEW_H

#include "src/config.h"
#include "src/presentation/observer.h"
#include "src/domain/animation/animationmodel.h"
#include "src/domain/animation/frame.h"

#include <qwidget.h>


struct SDL_Surface;

/**
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class FrameView : public QWidget, public Observer
{
	Q_OBJECT
public:
    FrameView(QWidget *parent=0, const char *name=0);
    ~FrameView();
	void setWidescreenRatio();
	void setNormalRatio();
	
	/**
	 *Function to recieven notification when a frame is added.
	 *
	 */
	void updateAdd(Frame*, unsigned int);
	
	/**
	 *Function to recieve notification when one or more frames are deleted.
	 *@param frameNumber the frame to remove
	 */
	void updateRemove(int);
	
	/**
	 *Function to recieve notification when one or more frames are moved.
	 *
	 */
	void updateMove(unsigned int fromPosition, unsigned int toPosition);
	
	/**
	 *Function to recieve notification when a new frame is selected.
	 */
	void updateNewActiveFrame(int frameNumber);
	
	/**
	 * Function to receive notification when the model is erased.
	 */
	void updateClear();
	
	/**
	 * Function to recieve notification when a frame is to be played.
	 * @param frameNumber the frame to be played
	 */
	void updatePlayFrame(int frameNumber);
	

protected:
	void resizeEvent(QResizeEvent *);
	void paintEvent(QPaintEvent *);
	
private:
	AnimationModel *animationModel;
	SDL_Surface *screen;
	SDL_Surface *currImage;
	int widthConst, heightConst;
	//int resizeWidth, resizeHeight;
	//int scaledWidth, scaledHeight;
	void setActiveFrame(const char *fileName);

};

#endif
