/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mainwindowgui.h"

#include "src/domain/domainfacade.h"
#include "src/application/runanimationhandler.h"
#include "picturepreview.h"

#include <stdlib.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qwizard.h>
#include <qevent.h>
#include <qdragobject.h> 
#include <qmessagebox.h>
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qiconset.h>
#include <qspinbox.h>
#include <qfiledialog.h>


MainWindowGUI::MainWindowGUI() : QMainWindow( 0, "Stopmotion", WDestructiveClose )
{
	centerWidget = NULL;
	frameView = NULL;
	frameBar = NULL;
	toolsMenu = NULL;
	workArea = NULL;
	homeDir = getenv("HOME");
	
	centerWidget = new QVBox(this);
	//Initializes and sets up the menu and the toolbar.
	makeMenues();
	//Initializes and sets up the framebar
	makeFrameBar(centerWidget);
	//Initializes and sets up the workarea consisting of a menu and the frameview.
	workArea = new QHBox(centerWidget);
	makeToolsMenu(workArea);
	makeFrameView(workArea);
	setCentralWidget(centerWidget);
	
	statusBar()->message( tr("Ready"), 2000 );
	this->setAcceptDrops(true);
}


MainWindowGUI::~MainWindowGUI()
{
	delete centerWidget;
}


void MainWindowGUI::makeMenues()
{
	QPopupMenu *fileMenu = new QPopupMenu(this);
	fileMenu->insertItem( tr("&New"), this, SLOT(newProject()), CTRL+Key_N );
	fileMenu->insertItem( tr("&Open"), this, SLOT(openProject()), CTRL+Key_O );
	fileMenu->insertSeparator();
	fileMenu->insertItem( tr("&Save"), this, SLOT(saveProject()), CTRL+Key_S, 1);
	fileMenu->insertItem( tr("Save &As"), this, SLOT(saveProject()), CTRL+Key_A);
	fileMenu->setItemEnabled(1, false);
	fileMenu->insertSeparator();
	fileMenu->insertItem( tr("Add &Frame"), this, SLOT(chooseFrame()), CTRL+Key_F );
	fileMenu->insertSeparator();
	fileMenu->insertItem( tr("&Quit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );
	menuBar()->insertItem( tr("&File"), fileMenu);
	
	
	QPopupMenu *editMenu = new QPopupMenu(this);
	editMenu->insertItem( tr("&Undo"), 1, CTRL+Key_Z );
	editMenu->setItemEnabled(1, false);
	editMenu->insertItem( tr("Re&do"), 2, CTRL+SHIFT+Key_Z );
	editMenu->setItemEnabled(2, false);
	editMenu->insertSeparator();
// 	editMenu->insertItem( tr("Cu&t"), 3, CTRL+Key_X );
// 	editMenu->setItemEnabled(3, false);
// 	editMenu->insertItem( tr("&Copy"), 4, CTRL+Key_C );
// 	editMenu->setItemEnabled(4, false);
	editMenu->insertItem( tr("&Paste"), 5, CTRL+Key_V );
	editMenu->setItemEnabled(5, false);
	menuBar()->insertItem( tr("&Edit"), editMenu);
	
	
	QPopupMenu *helpMenu = new QPopupMenu(this);
	helpMenu->insertItem(tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1 );
	helpMenu->insertSeparator();
	helpMenu->insertItem( tr("&About"), this, SLOT(showAboutDialog()) );
	menuBar()->insertItem( tr("&Help"), helpMenu);
	
	
	statusBar()->message( tr("Ready to rumble ;-)"), 2000 );
	statusBar()->setSizeGripEnabled(false);
}


void MainWindowGUI::makeFrameBar(QWidget * parent)
{
	frameBar = new FrameBar(parent);
	QString addFrameText = 
			tr("<h4>FrameBar</h4> "
			"<p>In this area you can see the frames "
			"in the animations and build the animation "
			"by moving the frames around.</p><p>You can "
			"switch to the next and the previous frame "
			"using the <b>arrow buttons</b> or <b>x</b> "
			"and <b>z</b></p> ");
    QWhatsThis::add( frameBar, addFrameText );
}

  
void MainWindowGUI::makeToolsMenu(QWidget * parent)
{
	toolsMenu = new QButtonGroup(1, Qt::Horizontal, parent);

	//Create widgets for toolsmenu:
	QPushButton *addFrameButton = new QPushButton( tr("&Add Frame"), toolsMenu );
	addFrameButton->setFocusPolicy ( QWidget::NoFocus );
	QString addFrameText = 
			tr("<h4>Add Frame</h4><p>Click this button to <em>add</em> a "
			"frame to the animation.</p>");
	QWhatsThis::add( addFrameButton, addFrameText );
	QObject::connect(addFrameButton, SIGNAL(clicked()), this, SLOT(chooseFrame()));

		
	QPushButton *removeFrameButton = new QPushButton( tr("&Remove Frame"), toolsMenu );
	removeFrameButton->setFocusPolicy ( QWidget::NoFocus );
	QString removeFrameText = 
			tr("<h4>Remove Frame</h4><p>Click this button to <em>remove</em> "
			"the selected frame from the animation.</p>");
	QWhatsThis::add( removeFrameButton, removeFrameText );
	QObject::connect(removeFrameButton, SIGNAL(clicked()), this, SLOT(removeFrame()));
	
	
	QPushButton *runAnimationButton = new QPushButton( tr("Run A&nimation"), toolsMenu );
	runAnimationButton->setFocusPolicy ( QWidget::NoFocus );	
	RunAnimationHandler *runAnimationHandler = new RunAnimationHandler( this,
			runAnimationButton, this->statusBar() );
	QString runAnimationText = 
			tr("<h4>Run Animation</h4><p>Click this button to <em>run</em> "
			"the animation in loops.</p>");
	QWhatsThis::add( runAnimationButton, runAnimationText );
	QObject::connect(runAnimationButton, SIGNAL(clicked()), runAnimationHandler, 
			SLOT(runAnimation()));
	
	
	QLabel *animationSpeedChooserCaption = new QLabel( tr("\nFPS chooser"), toolsMenu );
	QSpinBox *animationSpeedChooser = new QSpinBox( 1, 30, 1, toolsMenu );
	QString animationSpeedChooserText = 
			tr("<h4>FPS chooser</h4> "
			"<p>By changing the value in this "
			"chooser you set which speed the "
			"animation in the <b>FrameView</b> "
			"should run at.</p> "
			"<p>To start an animation press the "
			"<b>Run Animation</b> button.</p>");
    QWhatsThis::add( animationSpeedChooserCaption, animationSpeedChooserText );
    QWhatsThis::add( animationSpeedChooser, animationSpeedChooserText );
	QObject::connect( animationSpeedChooser, SIGNAL( valueChanged(int) ),
			runAnimationHandler, SLOT(setSpeed(int)) );
	animationSpeedChooser->setValue(10);
	
}


void MainWindowGUI::makeFrameView(QWidget *parent)
{
	frameView = new FrameView(parent);
	frameView->setMinimumSize(400, 300);
	frameView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	frameView->setBackgroundMode(NoBackground);
	QString frameViewText = 
			tr("<h4>FrameView</h4><p> In this area you can see the "
	 		"selected frame and the video from the "
			"camera so that you can take snapshots. You can also play "
			"animations in this window by pressing the "
			"<b>Run Animation</b> button.</p>");
    QWhatsThis::add( frameView, frameViewText );
}


//private slots
void MainWindowGUI::chooseFrame()
{
	fileDialog = new QFileDialog( homeDir, NULL, this );
	PicturePreview* p = new PicturePreview(fileDialog);
	
	fileDialog->setContentsPreviewEnabled( TRUE );
	fileDialog->setContentsPreview( p, p );
	fileDialog->setMode(QFileDialog::ExistingFiles);
	fileDialog->setPreviewMode( QFileDialog::Contents );
	fileDialog->setCaption( tr("Choose frames to add") );
	fileDialog->setFilters( QString(
					"Images (*.png *.jpg *.jpeg *.gif)\n"
					"Portable Network Graphics (*.png)\n"
					"Joint Photographic Ex. Gr. (*.jpg *.jpeg)\n"
					"GIMP native (*.xcf)\n"
					"Tagged Image File Format (*.tif)\n"
					"Windows Bitmap (*.bmp)\n"
					"TrueVision Targa (*.tga)\n"
					"Portable Anymap (*.pnm)\n"
					"X11 Pixmap (*.xpm)\n"
					"ZSoft IBM PC Paintbrush (*.pcx)\n"
					"CompuServe Graph. Interch. Format (*.gif)\n"
					"Interleaved Bitmap (*.lbm *.iff)\n"
					"All files (*)\n") );
	
	QObject::connect( fileDialog, SIGNAL(filesSelected (const QStringList &)), 
			this, SLOT(addFrames(const QStringList &)) );
	
	fileDialog->show();
}


void MainWindowGUI::addFrames( const QStringList & fileNames)
{
	Logger::get().logDebug("test");
	
	QStringList files = fileNames;
	
	if ( !files.isEmpty() ) {
		QStringList::Iterator it = files.begin();
		while( it != files.end() ) {
        	addFrame(*it);
        	++it;
    	}
	}
	else {
		statusBar()->message( tr("Adding aborted"), 2000 );
	}
}


void MainWindowGUI::addFrame( const QString &fileName )
{
	Logger::get().logDebug("Adding frame:");
	Logger::get().logDebug(fileName);
	
	
	char *f = (char*)fileName.ascii();

	DomainFacade::getFacade()->addFrame(f);
	
	statusBar()->message( tr("Added frame %1").arg(fileName), 2000 );
}


void MainWindowGUI::removeFrame()
{
	DomainFacade::getFacade()->removeFrame();
	statusBar()->message( tr("Removed the selected frame"), 2000 );
}


void MainWindowGUI::newProject() 
{
	DomainFacade::getFacade()->newProject();
}


void MainWindowGUI::openProject()
{
	QString file = QFileDialog::getOpenFileName(homeDir, "Stopmotion (*.sto)", NULL);
	if ( !file.isNull() ) {
		DomainFacade::getFacade()->openProject( file.ascii() );
	}
	else {
		// Wee.. Occours if the user press cancel
		Logger::get().logWarning("openDialog returned a NULL pointer");
	}
}


void MainWindowGUI::saveProject()
{
	QString dir = QFileDialog::getExistingDirectory(homeDir, NULL, NULL, 
						tr("Choose a directory to store project files in"), TRUE );
	QFileInfo *fi = new QFileInfo(dir);
	
	bool abort = false;
	while ( !fi->isWritable() && !abort ) {
		switch( QMessageBox::warning( NULL, tr("Warning"),
				tr("You do not have write permission to the directory.\n"
				"Press retry and select a another directory,\n"
				"or abort the save operation.\n"),
        		QMessageBox::Retry | QMessageBox::Default,
				QMessageBox::Abort | QMessageBox::Escape )) {
				
			case QMessageBox::Retry: // Retry clicked
				dir = QFileDialog::getExistingDirectory("/home", NULL, NULL, 
						tr("Choose a directory to store project files in"), TRUE );
				fi = new QFileInfo(dir);
				abort = fi->isWritable();
				break;
			case QMessageBox::Abort: // Abort clicked 
				abort = true;
				break;
		}
	}
	
	if (!abort) {
		DomainFacade::getFacade()->saveProject( dir.ascii() );
	}
}


void MainWindowGUI::saveAsProject()
{
	//DomainFacade::getFacade()->saveAsProject();
}


void MainWindowGUI::dragEnterEvent ( QDragEnterEvent * event)
{
	Logger::get().logDebug("Drag entering application");
	event->accept( QUriDrag::canDecode(event) );
}


void MainWindowGUI::dropEvent(QDropEvent *event)
{
	if( QUriDrag::canDecode(event) ) {
		QStringList fileNames;
		
		if( QUriDrag::decodeLocalFiles(event, fileNames) ) {
			for(unsigned int i=0; i<fileNames.size(); i++) {
				DomainFacade::getFacade()->addFrame( (char*)fileNames[i].ascii() );
			}
		}
	}
}


void MainWindowGUI::keyPressEvent( QKeyEvent *k )
{
	switch ( k->key() ) 
	{
		case Key_Right: case Key_X:
		{
			int afn = DomainFacade::getFacade()->getActiveFrameNumber();
			if(afn > -1 && afn < DomainFacade::getFacade()->getModelSize()-1) {
				DomainFacade::getFacade()->setActiveFrame(afn+1);
			}
			break;
		}
		case Key_Left: case Key_Z:
		{
			int afn = DomainFacade::getFacade()->getActiveFrameNumber();
			if(afn > 0) {
				DomainFacade::getFacade()->setActiveFrame(afn-1);
			}
			break;
		}
		default:
		{
			k->ignore();
			break;
		}
	}
}


void MainWindowGUI::showAboutDialog()
{
	QMessageBox *qb = new QMessageBox( tr("About stopmotion"), 
		tr("The stopmotion application for creating stopmotion animations."), 
		QMessageBox::NoIcon, QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape, 
		QMessageBox::NoButton, QMessageBox::NoButton, this );
	
	qb->setIconPixmap( QPixmap("graphics/updowntux.png") );
	qb->show();
}
