/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QTGUI_H
#define QTGUI_H

#include "src/config.h"
#include "framebar.h"
#include "frameview.h"

#include <qmainwindow.h>
#include <qevent.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qstring.h>
#include <qcolor.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qfiledialog.h>

/**
 *The main window class for the stopmotion application. 
 *This class sets up the main GUI and connects the buttons.
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
*/
class MainWindowGUI : public QMainWindow
{
	Q_OBJECT
public:
	/**
	 *Sets up the program main GUI and connect the buttons.
	 */
	MainWindowGUI();
	
	/**
	 *Cleans up and deletes the private member data.
	 */
	~MainWindowGUI();
	
private:
	
	QVBox *centerWidget;
	FrameView *frameView;
	FrameBar *frameBar;
	QButtonGroup *toolsMenu;
	QHBox *workArea;
	QFileDialog* fileDialog;
	const char *homeDir;
	
	/**
	 *Creates and sets up the menu and the toolbar.
	 */
	 void makeMenues();
	
	/**
	 *Creates and sets up the framebar.
	 *@param parent the widget the framebar will be inserted into.
	 */
	void makeFrameBar(QWidget * parent);
	
	/**
	 *Creates and sets up the frameview.
	 *@param parent the widget the frameview will be inserted into.
	 */
	void makeFrameView(QWidget * parent);
	
	/**
	 *Creates and sets up the toolsmenu.
	 *@param parent the widget the toolsmenu will be inserted into.
	 */
	void makeToolsMenu(QWidget * parent);
	
	/**
	 *Overloaded event listener for when when a drag enters the application.
	 *@param event information about the dragEnterEvent
	 */
	void dragEnterEvent ( QDragEnterEvent * event);
	
	/**
	 *Overloaded event listener for when a drop event occur in the application.
	 *@param event information about the dropEvent
	 */
	void dropEvent(QDropEvent *event);
	
	
	/**
	 * Overloaded event listener which recieves information when a keyboard key is
	 * pressed.
	 * @param k information about the key event. 
	 */
	void keyPressEvent( QKeyEvent *k );
	
private slots:
	/**
	 *Brings up a dialog so that the user can choose a file to load.
	 */
	void chooseFrame();
	
	void addFrames(const QStringList & fileNames);
	
	/**
	 *Adds the file fileName to the animation and calls notify.
	 *@param fileName the file to be added to the animation.
	 */
    void addFrame( const QString &fileName );
	
	/**
	 *Removes the currently selected frame from the animation.
	 */
	void removeFrame();
	
	void newProject();
	void openProject();
	void saveProject();
	void saveAsProject();
	
	/**
	 * Brings up an about dialog with information about the application.
	 */
	void showAboutDialog();
};

#endif
