/***************************************************************************
 *   Copyright (C) 2005 by fredrik                                         *
 *   fredrik@Elanor                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "thumbview.h"

#include "src/domain/domainfacade.h"

#include <qapplication.h>
#include <qdragobject.h> 


ThumbView::ThumbView(QWidget *parent, int frameNumber) 
 : QLabel(parent)
{
	this->frameNumber = frameNumber;
	this->setAcceptDrops(true);
}


ThumbView::~ThumbView()
{
}


void ThumbView::mousePressEvent( QMouseEvent * e )
{
	DomainFacade::getFacade()->setActiveFrame(frameNumber);
	dragPos = e->pos();
}


void ThumbView::mouseMoveEvent(QMouseEvent *me)
{
	if(me->state() & LeftButton) {
		int distance = (me->pos() - dragPos).manhattanLength();
		if(distance > QApplication::startDragDistance()) {
			startDrag();
		}
	}
	QLabel::mouseMoveEvent(me);
}


void ThumbView::setFrameNumber(int frameNumber)
{
	this->frameNumber = frameNumber;
}


int ThumbView::getFrameNumber()
{
	return frameNumber;
}


void ThumbView::startDrag()
{
	QStrList lst;
	lst.append( DomainFacade::getFacade()->getFrame( this->frameNumber )->getPicture() );
	QUriDrag *drag = new QUriDrag( lst, this );
	drag->setPixmap(*pixmap());
	drag->drag();
}


void ThumbView::dragEnterEvent ( QDragEnterEvent * event)
{
	Logger::get().logDebug("Drag entering the ThumbView");
	event->accept( QUriDrag::canDecode(event) );
}


void ThumbView::dropEvent(QDropEvent *event)
{
	
	if(event->source() != 0) {
		Logger::get().logDebug("Moving picture");
		DomainFacade::getFacade()->moveFrame( ((ThumbView*)event->source())->getFrameNumber(), this->frameNumber );
	}
	else {
		DomainFacade::getFacade()->setActiveFrame(this->frameNumber);
	
		if( QUriDrag::canDecode(event) ) {
			QStringList fileNames;
			
			if( QUriDrag::decodeLocalFiles(event, fileNames) ) {
				for(unsigned int i=0; i<fileNames.size(); i++) {
					DomainFacade::getFacade()->addFrame( (char*)fileNames[i].ascii() );
				}
			}
		}
	}
}
