/***************************************************************************
 *   Copyright (C) 2005 by fredrik                                         *
 *   fredrik@Elanor                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THUMBVIEW_H
#define THUMBVIEW_H

#include <qlabel.h>


/**
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class ThumbView : public QLabel {
public:
	ThumbView(QWidget *parent = 0, int frameNumber = 0);
	~ThumbView();

	
	/**
	 *Overloaded event function to recieve mousepress-events.
	 *@param e information about the mouseevent.
	 */
	void mousePressEvent( QMouseEvent * e );
	
	
	/**
	 *Overloaded event function to recieve mouseMoveEvents.
	 *@param me information about the mouseMoveEvent. 
	 */
	void mouseMoveEvent(QMouseEvent *me);
	
	
	/**
	 *Function to set the frameNumber when the frame change position.
	 *@param frameNumber the new frameNumber for the frame.
	 */
	void setFrameNumber(int frameNumber);
	
	
	/**
	 *Overloaded event function for when a drop event occur in the thumbview.
	 *@param event information about the dropEvent
	 */
	void dropEvent(QDropEvent *event);
	
	
	/**
	 *Overloaded event function for when when a drag enters the ThumbView.
	 *@param event information about the dragEnterEvent
	 */
	void dragEnterEvent ( QDragEnterEvent * event);

	
	/**
	 *Retrieves the frameNumber for this ThumbView
	 *@return the frameNumber for this ThumbView
	 */
	int getFrameNumber();
	
private:
	/**The number this frame has in the framebar*/
	int frameNumber;
	
	/**Coordinate for calculating when a drag should start to give some slack*/
	QPoint dragPos;
	
	/**
	 *Starts an uri drag of the picture in this label.
	 */
	void startDrag();
};

#endif
