/***************************************************************************
 *   Copyright (C) 2005 by Bjoern Erik Nilsen & Fredrik Berg Kjoelstad     *
 *   bjoern_erik_nilsen@hotmail.com & fredrikbk@hotmail.com                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef OBSERVER_H
#define OBSERVER_H

#include "../config.h"
#include "../domain/animation/frame.h"

/**
 *The observer interface. All classes who wants to be notified when something 
 *changes in the animationmodel has to implement from this class.
 *
 *The observers implemented with this class also has to be attatched to the
 *animationmodel.
 *
 *The observer is implemented with strong use of the push model. Although this
 *decreases the flexibility we gain alot in efficiency which is more
 *important to us.
 *
 *@author Bjoern Erik Nilsen & Fredrik Berg Kjoelstad
 */
class Observer
{
public:
	/**
	 * Abstract function for recieving information about new frames added to the model.
	 * 
	 * @param frame the frame which has been added to the model.
	 * @param index the place the frame has been added
	 */
	virtual void updateAdd(Frame* frame, unsigned int index) = 0;
	
	
	/**
	 * Abstract function for recieving information about frames removed from the model.
	 * 
	 * @param frameNumber the number of the frame which has been removed.
	 */
	virtual void updateRemove(int frameNumber) = 0;
	
	
	/**
	 * Abstract function for recieving information when frames are moved in the model.
	 * @param fromPosition the place the frame has been moved from.
	 * @param toPosition the place the frame has been moved to.
	 */
	virtual void updateMove(unsigned int fromPosition, unsigned int toPosition) = 0;
	

	/**
	 * Abstract function for recieving information when a new frame are selected.
	 * @param frameNumber the new active frame.
	 */
	virtual void updateNewActiveFrame(int frameNumber) = 0;
	
	
	/**
	 * Abstract function for receiving information when the model is erased.
	 */
	virtual void updateClear() = 0;
	
	
	/**
	 * Abstract function for recieving information when a frame is to be played.
	 * @param frameNumber the frame to be played
	 */
	virtual void updatePlayFrame(int frameNumber) = 0;
};

#endif
